package aliyun

import (
	"encoding/json"
	"errors"
	"fmt"
	dypnsapi20170525 "github.com/alibabacloud-go/dypnsapi-20170525/v2/client"
	util "github.com/alibabacloud-go/tea-utils/v2/service"
	"github.com/alibabacloud-go/tea/tea"
	"strings"
)

// 一键登录取号
func AliyunFastPhone(accessKeyId, accessKeySecret, token string) (string, error) {
	client, _err := CreateClient(accessKeyId, accessKeySecret)
	if _err != nil {
		return "", _err
	}
	getMobileRequest := &dypnsapi20170525.GetMobileRequest{
		AccessToken: tea.String(token),
	}
	runtime := &util.RuntimeOptions{}
	phone, tryErr := func() (phone string, _e error) {
		defer func() {
			if r := tea.Recover(recover()); r != nil {
				_e = r
			}
		}()
		// 复制代码运行请自行打印 API 的返回值
		_result, _err := client.GetMobileWithOptions(getMobileRequest, runtime)
		if _err != nil {
			return "", _err
		}
		phone = *_result.Body.GetMobileResultDTO.Mobile
		return phone, nil
	}()

	if tryErr != nil {
		var error = &tea.SDKError{}
		if _t, ok := tryErr.(*tea.SDKError); ok {
			error = _t
		} else {
			error.Message = tea.String(tryErr.Error())
		}
		// 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
		// 错误 message
		fmt.Println(tea.StringValue(error.Message))
		// 诊断地址
		var data interface{}
		d := json.NewDecoder(strings.NewReader(tea.StringValue(error.Data)))
		d.Decode(&data)
		if m, ok := data.(map[string]interface{}); ok {
			recommend, _ := m["Recommend"]
			fmt.Println(recommend)
		}
		_, _err = util.AssertAsString(error.Message)
		if _err != nil {
			return "", _err
		}
		return "", errors.New("获取手机号失败")
	}
	return phone, _err
}

// 本机号码校验
func AliyunCheckFastPhone(accessKeyId, accessKeySecret, token, phone string) (string, error) {
	client, _err := CreateClient(accessKeyId, accessKeySecret)
	if _err != nil {
		return "", _err
	}
	getMobileRequest := &dypnsapi20170525.VerifyMobileRequest{
		AccessCode:  tea.String(token),
		PhoneNumber: tea.String(phone),
	}
	runtime := &util.RuntimeOptions{}
	state, tryErr := func() (state string, _e error) {
		defer func() {
			if r := tea.Recover(recover()); r != nil {
				_e = r
			}
		}()
		// 复制代码运行请自行打印 API 的返回值
		_result, _err := client.VerifyMobileWithOptions(getMobileRequest, runtime)
		if _err != nil {
			return "", _err
		}
		state = *_result.Body.GateVerifyResultDTO.VerifyResult
		return state, nil
	}()

	if tryErr != nil {
		var error = &tea.SDKError{}
		if _t, ok := tryErr.(*tea.SDKError); ok {
			error = _t
		} else {
			error.Message = tea.String(tryErr.Error())
		}
		// 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
		// 错误 message
		fmt.Println(tea.StringValue(error.Message))
		// 诊断地址
		var data interface{}
		d := json.NewDecoder(strings.NewReader(tea.StringValue(error.Data)))
		d.Decode(&data)
		if m, ok := data.(map[string]interface{}); ok {
			recommend, _ := m["Recommend"]
			fmt.Println(recommend)
		}
		_, _err = util.AssertAsString(error.Message)
		if _err != nil {
			return "", _err
		}
		return "", errors.New("校验失败")
	}
	return state, _err
}
