package egg_energy

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/enum"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/md"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule"
	enum2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/enum"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/svc"
	egg_system_rules "code.fnuoos.com/EggPlanet/egg_system_rules.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/EggPlanet/egg_system_rules.git/utils/logx"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"fmt"
	"github.com/shopspring/decimal"
	"xorm.io/xorm"
)

// ActivityCoinStartExchangeEggEnergy 蛋蛋积分【开始】兑换成蛋蛋能量
func ActivityCoinStartExchangeEggEnergy(engine *xorm.Engine, req md.EggEnergyStructForStartExchangeGreenEnergy, ch *rabbit.Channel) (err error) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()

	//1、查询sign 记录
	eggSignInDb := implement.NewEggSignInDb(engine)
	sign, err := eggSignInDb.EggSignInGet(req.SignId)
	if err != nil {
		return err
	}

	//2、扣除用户对应蛋蛋活跃积分数量
	err = rule.DealUserVirtualCoin(session, md2.DealUserVirtualCoinReq{
		Kind:         "sub",
		Title:        enum.EggPointCoinToExchangeToPersonEggEnergy.String(),
		TransferType: int(enum.EggPointCoinToExchangeToPersonEggEnergy),
		CoinId:       req.PersonPointCoinId,
		Uid:          req.Uid,
		Amount:       egg_system_rules.StrToFloat64(sign.TotalPersonEggPoints),
	})
	if err != nil {
		fmt.Println("ActivityCoinStartExchangeEggEnergy:::::err111:::", err)
		_ = session.Rollback()
		return
	}

	err = rule.DealUserVirtualCoin(session, md2.DealUserVirtualCoinReq{
		Kind:         "sub",
		Title:        enum.EggPointCoinToExchangeToTeamEggEnergy.String(),
		TransferType: int(enum.EggPointCoinToExchangeToTeamEggEnergy),
		CoinId:       req.TeamEnergyCoinId,
		Uid:          req.Uid,
		Amount:       egg_system_rules.StrToFloat64(sign.TotalTeamEggPoints),
	})
	if err != nil {
		fmt.Println("ActivityCoinStartExchangeEggEnergy:::::er222:::", err)
		_ = session.Rollback()
		return
	}

	//3、进行兑换购买
	eggEnergyCoreData, cb, err := svc.GetEggEnergyCoreData(engine)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}
	//3.1、计算涨价公式
	err1, calcPriceIncreaseFormulaRespForPerson := CalcPriceIncreaseFormula(sign.TotalPersonEggPointsAmountValue, eggEnergyCoreData)
	if err1 != nil {
		_ = session.Rollback()
		return err1
	}

	err1, calcPriceIncreaseFormulaRespForTeam := CalcPriceIncreaseFormula(sign.TotalPersonEggPointsAmountValue, eggEnergyCoreData)
	if err1 != nil {
		_ = session.Rollback()
		return err1
	}

	//3.2、更改动态数据
	err = DealAvailableEggEnergyCoin(session, int(enum2.PersonalActivePointRedemption), eggEnergyCoreData, md.DealAvailableEggEnergyCoinReq{
		Amount:                 calcPriceIncreaseFormulaRespForPerson.GetEggEnergyNums,
		AmountFee:              "",
		BeforePrice:            calcPriceIncreaseFormulaRespForPerson.BeforePrice,
		AfterPrice:             calcPriceIncreaseFormulaRespForPerson.AfterPrice,
		BeforePlanetTotalValue: calcPriceIncreaseFormulaRespForPerson.BeforePlanetTotalValue,
		AfterPlanetTotalValue:  calcPriceIncreaseFormulaRespForPerson.AfterPlanetTotalValue,
		BeforeEnergyTotalNums:  calcPriceIncreaseFormulaRespForPerson.BeforeEnergyTotalNums,
		AfterEnergyTotalNums:   calcPriceIncreaseFormulaRespForPerson.AfterEnergyTotalNums,
	})
	if err != nil {
		fmt.Println("ActivityCoinStartExchangeEggEnergy:::::er333:::", err)
		_ = session.Rollback()
		return err
	}
	err = DealAvailableEggEnergyCoin(session, int(enum2.TeamActivePointRedemption), eggEnergyCoreData, md.DealAvailableEggEnergyCoinReq{
		Amount:                 calcPriceIncreaseFormulaRespForTeam.GetEggEnergyNums,
		AmountFee:              "",
		BeforePrice:            calcPriceIncreaseFormulaRespForTeam.BeforePrice,
		AfterPrice:             calcPriceIncreaseFormulaRespForTeam.AfterPrice,
		BeforePlanetTotalValue: calcPriceIncreaseFormulaRespForTeam.BeforePlanetTotalValue,
		AfterPlanetTotalValue:  calcPriceIncreaseFormulaRespForTeam.AfterPlanetTotalValue,
		BeforeEnergyTotalNums:  calcPriceIncreaseFormulaRespForTeam.BeforeEnergyTotalNums,
		AfterEnergyTotalNums:   calcPriceIncreaseFormulaRespForTeam.AfterEnergyTotalNums,
	})
	if err != nil {
		fmt.Println("ActivityCoinStartExchangeEggEnergy:::::err444:::", err)
		_ = session.Rollback()
		return err
	}

	//4、修改sign记录
	startTime, _ := egg_system_rules.StringToTime(sign.StartTime)
	endTime, _ := egg_system_rules.StringToTime(sign.EndTime)
	seconds := endTime.Sub(startTime).Seconds()
	secondsValue := decimal.NewFromFloat(seconds)
	eggEnergyNumsForPersonValue, _ := decimal.NewFromString(calcPriceIncreaseFormulaRespForPerson.GetEggEnergyNums)
	eggEnergyNumsForTeamValue, _ := decimal.NewFromString(calcPriceIncreaseFormulaRespForTeam.GetEggEnergyNums)
	sign.EstimatePerSecondPersonEggEnergyValue = eggEnergyNumsForPersonValue.Div(secondsValue).String()
	sign.EstimatePerSecondTeamEggEnergyValue = eggEnergyNumsForTeamValue.Div(secondsValue).String()
	_, err = eggSignInDb.EggSignInUpdateBySession(session, sign.Id, sign, "estimate_per_second_person_egg_energy_value", "estimate_per_second_team_egg_energy_value")
	if err != nil {
		fmt.Println("ActivityCoinStartExchangeEggEnergy:::::err555:::", err)
		_ = session.Rollback()
		return err
	}

	err = session.Commit()
	if err != nil {
		fmt.Println("ActivityCoinStartExchangeEggEnergy:::::err666:::", err)
		_ = session.Rollback()
		return err
	}

	//TODO::推入rabbitmq 异步处理
	ch.Publish(md.EggEnergyExchange, md.EggEnergyStructForAutoExchangeGreenEnergy{
		Uid:                   req.Uid,
		SignId:                sign.Id,
		PersonEggEnergyCoinId: req.PersonEnergyCoinId,
		TeamEggEnergyCoinId:   req.TeamEggEnergyCoinId,
		StartExchangeTime:     sign.StartTime,
		EndExchangeTime:       sign.EndTime,
	}, md.EggEnergyRoutKeyForAutoExchangeGreenEnergy)

	return
}
